import unittest
import random
import time
import math
import sys

import cv

class FrameInterpolator:
  def __init__(self, prev, curr):

    w,h = cv.GetSize(prev)

    self.offx = cv.CreateMat(h, w, cv.CV_32FC1)
    self.offy = cv.CreateMat(h, w, cv.CV_32FC1)
    for y in range(h):
      for x in range(w):
        self.offx[y,x] = x
        self.offy[y,x] = y

    self.maps = [ None, None ]
    for i,a,b in [ (0, prev, curr), (1, curr, prev) ]:
      velx = cv.CreateMat(h, w, cv.CV_32FC1)
      vely = cv.CreateMat(h, w, cv.CV_32FC1)
      cv.CalcOpticalFlowLK(a, b, (15,15), velx, vely)

      for j in range(10):
        cv.Smooth(velx, velx, param1 = 7)
        cv.Smooth(vely, vely, param1 = 7)
      self.maps[i] = (velx, vely)

  def lerp(self, t, prev, curr):

    w,h = cv.GetSize(prev)

    x = cv.CreateMat(h, w, cv.CV_32FC1)
    y = cv.CreateMat(h, w, cv.CV_32FC1)
    d = cv.CloneImage(prev)
    d0 = cv.CloneImage(prev)
    d1 = cv.CloneImage(prev)

    # d0 is curr mapped backwards in time, so 1.0 means exacty curr
    velx,vely = self.maps[0]
    cv.ConvertScale(velx, x, 1.0 - t)
    cv.ConvertScale(vely, y, 1.0 - t)
    cv.Add(x, self.offx, x)
    cv.Add(y, self.offy, y)
    cv.Remap(curr, d0, x, y)

    # d1 is prev mapped forwards in time, so 0.0 means exactly prev
    velx,vely = self.maps[1]
    cv.ConvertScale(velx, x, t)
    cv.ConvertScale(vely, y, t)
    cv.Add(x, self.offx, x)
    cv.Add(y, self.offy, y)
    cv.Remap(prev, d1, x, y)

    cv.AddWeighted(d0, t, d1, 1.0 - t, 0.0, d)
    return d

class TestDirected(unittest.TestCase):

  depths = [ cv.IPL_DEPTH_8U, cv.IPL_DEPTH_8S, cv.IPL_DEPTH_16U, cv.IPL_DEPTH_16S, cv.IPL_DEPTH_32S, cv.IPL_DEPTH_32F, cv.IPL_DEPTH_64F ]

  mat_types = [
    cv.CV_8UC1,
    cv.CV_8UC2,
    cv.CV_8UC3,
    cv.CV_8UC4,
    cv.CV_8SC1,
    cv.CV_8SC2,
    cv.CV_8SC3,
    cv.CV_8SC4,
    cv.CV_16UC1,
    cv.CV_16UC2,
    cv.CV_16UC3,
    cv.CV_16UC4,
    cv.CV_16SC1,
    cv.CV_16SC2,
    cv.CV_16SC3,
    cv.CV_16SC4,
    cv.CV_32SC1,
    cv.CV_32SC2,
    cv.CV_32SC3,
    cv.CV_32SC4,
    cv.CV_32FC1,
    cv.CV_32FC2,
    cv.CV_32FC3,
    cv.CV_32FC4,
    cv.CV_64FC1,
    cv.CV_64FC2,
    cv.CV_64FC3,
    cv.CV_64FC4,
  ]

  def depthsize(self, d):
    return { cv.IPL_DEPTH_8U : 1,
             cv.IPL_DEPTH_8S : 1,
             cv.IPL_DEPTH_16U : 2,
             cv.IPL_DEPTH_16S : 2,
             cv.IPL_DEPTH_32S : 4,
             cv.IPL_DEPTH_32F : 4,
             cv.IPL_DEPTH_64F : 8 }[d]

  def expect_exception(self, func, exception):
    tripped = False
    try:
      func()
    except exception:
      tripped = True
    self.assert_(tripped)

  def test_LoadImage(self):
    self.expect_exception(lambda: cv.LoadImage(), TypeError)
    self.expect_exception(lambda: cv.LoadImage(4), TypeError)
    self.expect_exception(lambda: cv.LoadImage('foo.jpg', 1, 1), TypeError)
    self.expect_exception(lambda: cv.LoadImage('foo.jpg', xiscolor=cv.CV_LOAD_IMAGE_COLOR), TypeError)

  def test_CreateMat(self):
    for rows in [2, 4, 16, 64, 512, 640]: # XXX - 1 causes bug in OpenCV
      for cols in [1, 2, 4, 16, 64, 512, 640]:
        for t in self.mat_types:
          m = cv.CreateMat(rows, cols, t)

  def test_CreateImage(self):
    for w in [ 1, 4, 64, 512, 640]:
      for h in [ 1, 4, 64, 480, 512]:
        for c in [1, 2, 3, 4]:
          for d in self.depths:
            a = cv.CreateImage((w,h), d, c);
            self.assert_(a.width == w)
            self.assert_(a.height == h)
            self.assert_(a.nChannels == c)
            self.assert_(a.depth == d)
            self.assert_(cv.GetSize(a) == (w, h))
            # self.assert_(cv.GetElemType(a) == d)

  def test_types(self):
    self.assert_(type(cv.CreateImage((7,5), cv.IPL_DEPTH_8U, 1)) == cv.iplimage)
    self.assert_(type(cv.CreateMat(5, 7, cv.CV_32FC1)) == cv.cvmat)

  def test_GetSize(self):
    self.assert_(cv.GetSize(cv.CreateMat(5, 7, cv.CV_32FC1)) == (7,5))
    self.assert_(cv.GetSize(cv.CreateImage((7,5), cv.IPL_DEPTH_8U, 1)) == (7,5))

  def test_GetAffineTransform(self):
    mapping = cv.CreateMat(2, 3, cv.CV_32FC1)
    cv.GetAffineTransform([ (0,0), (1,0), (0,1) ], [ (0,0), (17,0), (0,17) ], mapping)
    self.assertAlmostEqual(mapping[0,0], 17, 2)
    self.assertAlmostEqual(mapping[1,1], 17, 2)

  def test_MinMaxLoc(self):
    scribble = cv.CreateImage((640,480), cv.IPL_DEPTH_8U, 1)
    los = [ (random.randrange(480), random.randrange(640)) for i in range(100) ]
    his = [ (random.randrange(480), random.randrange(640)) for i in range(100) ]
    for (lo,hi) in zip(los,his):
      cv.Set(scribble, 128)
      scribble[lo] = 0
      scribble[hi] = 255
      r = cv.MinMaxLoc(scribble)
      self.assert_(r == (0, 255, tuple(reversed(lo)), tuple(reversed(hi))))

  def failing_test_exception(self):
    a = cv.CreateImage((640,480), cv.IPL_DEPTH_8U, 1)
    b = cv.CreateImage((640,480), cv.IPL_DEPTH_8U, 1)
    self.expect_exception(lambda: cv.Laplace(a, b), cv.error)

  def test_tostring(self):
    for w in [ 1, 4, 64, 512, 640]:
      for h in [ 1, 4, 64, 480, 512]:
        for c in [1, 2, 3, 4]:
          for d in self.depths:
            a = cv.CreateImage((w,h), d, c);
            self.assert_(len(a.tostring()) == w * h * c * self.depthsize(d))

  def test_depths(self):
    """ Make sure that the depth enums are unique """
    self.assert_(len(self.depths) == len(set(self.depths)))

  def test_leak(self):
    """ If CreateImage is not releasing image storage, then the loop below should use ~4GB of memory. """
    for i in range(4000):
      a = cv.CreateImage((1024,1024), cv.IPL_DEPTH_8U, 1)

  def test_voronoi(self):
    w,h = 500,500

    storage = cv.CreateMemStorage(0)

    def facet_edges(e0):
      e = e0
      while True:
        e = cv.Subdiv2DGetEdge(e, cv.CV_NEXT_AROUND_LEFT)
        yield e
        if e == e0:
          break

    def areas(edges):
      seen = []
      seensorted = []
      for edge in edges:
        pts = [ cv.Subdiv2DEdgeOrg(e) for e in facet_edges(edge) ]
        if not (None in pts):
          l = [p.pt for p in pts]
          ls = sorted(l)
          if not(ls in seensorted):
            seen.append(l)
            seensorted.append(ls)
      return seen

    for npoints in range(1, 200):
      points = [ (random.randrange(w), random.randrange(h)) for i in range(npoints) ]
      subdiv = cv.CreateSubdivDelaunay2D( (0,0,w,h), storage )
      for p in points:
        cv.SubdivDelaunay2DInsert( subdiv, p)
      cv.CalcSubdivVoronoi2D(subdiv)
      as = areas([ cv.Subdiv2DRotateEdge(e, 1) for e in subdiv.edges ] + [ cv.Subdiv2DRotateEdge(e, 3) for e in subdiv.edges ])
      self.assert_(len(as) == len(set(points)))

      if False:
        img = cv.CreateImage((w,h), cv.IPL_DEPTH_8U, 3)
        cv.SetZero(img)
        def T(x): return int(x) # int(300+x/16)
        for pts in as:
          cv.FillConvexPoly( img, [(T(x),T(y)) for (x,y) in pts], cv.RGB(100+random.randrange(156),random.randrange(256),random.randrange(256)), cv.CV_AA, 0 );
        for x,y in points:
          cv.Circle(img, (T(x), T(y)), 3, cv.RGB(0,0,0), -1)

        cv.ShowImage("snap", img)
        if cv.WaitKey(10) > 0:
          break

  def test_lineclip(self):
    w,h = 640,480
    img = cv.CreateImage((w,h), cv.IPL_DEPTH_8U, 1)
    cv.SetZero(img)
    tricky = [ -8000, -2, -1, 0, 1, h/2, h-1, h, h+1, w/2, w-1, w, w+1, 8000]
    for x0 in tricky:
      for y0 in tricky:
        for x1 in tricky:
          for y1 in tricky:
            for thickness in [ 0, 1, 8 ]:
              for line_type in [0, 4, 8, cv.CV_AA ]:
                cv.Line(img, (x0,y0), (x1,y1), 255, thickness, line_type)
    # just check that something was drawn
    self.assert_(cv.Sum(img)[0] > 0)

  def test_GetSubRect(self):
    src = cv.CreateImage((100,100), 8, 1)
    data = "z" * (100 * 100)

    cv.SetData(src, data, 100)
    start_count = sys.getrefcount(data)

    iter = 77
    subs = []
    for i in range(iter):
      sub = cv.CreateMat(10, 10, cv.CV_8UC1)
      cv.GetSubRect(src, sub, (0, 0, 10, 10))
      subs.append(sub)
    self.assert_(sys.getrefcount(data) == (start_count + iter))

    src = cv.LoadImage("lena.jpg", 0)
    made = cv.CreateImage(cv.GetSize(src), 8, 1)
    sub = cv.CreateMat(32, 32, cv.CV_8UC1)
    for x in range(0, 512, 32):
      for y in range(0, 512, 32):
        cv.GetSubRect(src, sub, (x, y, 32, 32))
        cv.SetImageROI(made, (x, y, 32, 32))
        cv.Copy(sub, made)
    cv.ResetImageROI(made)
    cv.AbsDiff(made, src, made)
    self.assert_(cv.CountNonZero(made) == 0)

  def assertSame(self, a, b):
    w,h = cv.GetSize(a)
    d = cv.CreateMat(h, w, cv.CV_8UC1)
    cv.AbsDiff(a, b, d)
    self.assert_(cv.CountNonZero(d) == 0)

  def test_GetStarKeypoints(self):
    src = cv.LoadImage("im640x480.pgm", 0)
    storage = cv.CreateMemStorage()
    kp = cv.GetStarKeypoints(src, storage)
    self.assert_(len(kp) > 0)
    for (x,y),scale,r in kp:
      self.assert_(0 <= x)
      self.assert_(x <= cv.GetSize(src)[0])
      self.assert_(0 <= y)
      self.assert_(y <= cv.GetSize(src)[1])
    return
    scribble = cv.CreateImage(cv.GetSize(src), 8, 3)
    cv.CvtColor(src, scribble, cv.CV_GRAY2BGR)
    for (x,y),scale,r in kp:
      print x,y,scale,r
      cv.Circle(scribble, (x,y), scale, cv.RGB(255,0,0))
    self.snap(scribble)

  def test_Threshold(self):
    """ directed test for bug 2790622 """
    src = cv.LoadImage("lena.jpg", 0)
    results = set()
    for i in range(10):
      dst = cv.CreateImage(cv.GetSize(src), cv.IPL_DEPTH_8U, 1)
      cv.Threshold(src, dst, 128, 128, cv.CV_THRESH_BINARY)
      results.add(dst.tostring())
    # Should have produced the same answer every time, so results set should have size 1
    self.assert_(len(results) == 1)

  def failing_test_Circle(self):
    """ smoke test to draw circles, many clipped """
    for w,h in [(2,77), (77,2), (256, 256), (640,480)]:
      img = cv.CreateImage((w,h), cv.IPL_DEPTH_8U, 1)
      cv.SetZero(img)
      tricky = [ -8000, -2, -1, 0, 1, h/2, h-1, h, h+1, w/2, w-1, w, w+1, 8000]
      for x0 in tricky:
        for y0 in tricky:
          for r in [ 0, 1, 2, 3, 4, 5, w/2, w-1, w, w+1, h/2, h-1, h, h+1, 8000 ]:
            for thick in [1, 2, 10]:
              for t in [0, 8, 4, cv.CV_AA]:
                cv.Circle(img, (x0,y0), r, 255, thick, t)
    # just check that something was drawn
    self.assert_(cv.Sum(img)[0] > 0)

  def test_text(self):
    img = cv.CreateImage((640,40), cv.IPL_DEPTH_8U, 1)
    cv.SetZero(img)
    font = cv.InitFont(cv.CV_FONT_HERSHEY_SIMPLEX, 1, 1)
    message = "XgfooX"
    cv.PutText(img, message, (320,30), font, 255)
    ((w,h),bl) = cv.GetTextSize(message, font)

    # Find nonzero in X and Y
    Xs = []
    for x in range(640):
      cv.SetImageROI(img, (x, 0, 1, 40))
      Xs.append(cv.Sum(img)[0] > 0)
    def firstlast(l):
      return (l.index(True), len(l) - list(reversed(l)).index(True))

    Ys = []
    for y in range(40):
      cv.SetImageROI(img, (0, y, 640, 1))
      Ys.append(cv.Sum(img)[0] > 0)

    x0,x1 = firstlast(Xs)
    y0,y1 = firstlast(Ys)
    actual_width = x1 - x0
    actual_height = y1 - y0

    # actual_width can be up to 8 pixels smaller than GetTextSize says
    self.assert_(actual_width <= w)
    self.assert_((w - actual_width) <= 8)

    # actual_height can be up to 4 pixels smaller than GetTextSize says
    self.assert_(actual_height <= (h + bl))
    self.assert_(((h + bl) - actual_height) <= 4)

    cv.ResetImageROI(img)
    self.assert_(w != 0)
    self.assert_(h != 0)

  def failing_test_sizes(self):
    sizes = [ 1, 2, 3, 97, 255, 256, 257, 947 ]
    for w in sizes:
      for h in sizes:
        # Create an IplImage 
        im = cv.CreateImage((w,h), cv.IPL_DEPTH_8U, 1)
        cv.Set(im, 1)
        self.assert_(cv.Sum(im)[0] == (w * h))
        del im
        # Create a CvMat
        mt = cv.CreateMat(h, w, cv.CV_8UC1)
        cv.Set(mt, 1)
        self.assert_(cv.Sum(mt)[0] == (w * h))

    random.seed(7)
    for dim in range(1, cv.CV_MAX_DIM + 1):
      for attempt in range(10):
        dims = [ random.choice([1,1,1,1,2,3]) for i in range(dim) ]
        mt = cv.CreateMatND(dims, cv.CV_8UC1)
        cv.SetZero(mt)
        self.assert_(cv.Sum(mt)[0] == 0)
        # Set to all-ones, verify the sum
        cv.Set(mt, 1)
        expected = 1
        for d in dims:
          expected *= d
        self.assert_(cv.Sum(mt)[0] == expected)

  def test_random(self):
    seeds = [ 0, 1, 2**48, 2**48 + 1 ]
    sequences = set()
    for s in seeds:
      rng = cv.RNG(s)
      sequences.add(str([cv.RandInt(rng) for i in range(10)]))
    self.assert_(len(seeds) == len(sequences))
      
    rng = cv.RNG(0)
    im = cv.CreateImage((1024,1024), cv.IPL_DEPTH_8U, 1)
    cv.RandArr(rng, im, cv.CV_RAND_UNI, 0, 256)
    cv.RandArr(rng, im, cv.CV_RAND_NORMAL, 128, 30)
    # XXX TODO - blocked by failing_test_sizes
    if 0:
      hist = cv.CreateHist([ 256 ], cv.CV_HIST_ARRAY, [ (0,255) ], 1)
      cv.CalcHist([im], hist)

    rng = cv.RNG()
    for i in range(1000):
      v = cv.RandReal(rng)
      self.assert_(0 <= v)
      self.assert_(v < 1)

  def test_access(self):
    cnames = { 1:cv.CV_32FC1, 2:cv.CV_32FC2, 3:cv.CV_32FC3, 4:cv.CV_32FC4 }

    for w in range(1,11):
      for h in range(2,11):
        for c in [1,2]:
          for o in [ cv.CreateMat(h, w, cnames[c]), cv.CreateImage((w,h), cv.IPL_DEPTH_32F, c) ][1:]:
            pattern = [ (i,j) for i in range(w) for j in range(h) ]
            random.shuffle(pattern)
            for k,(i,j) in enumerate(pattern):
              if c == 1:
                o[j,i] = k
              else:
                o[j,i] = (k,) * c
            for k,(i,j) in enumerate(pattern):
              if c == 1:
                self.assert_(o[j,i] == k)
              else:
                self.assert_(o[j,i] == (k,)*c)

  def test_InitLineIterator(self):
    scribble = cv.CreateImage((640,480), cv.IPL_DEPTH_8U, 1)
    self.assert_(len(list(cv.InitLineIterator(scribble, (20,10), (30,10)))) == 11)

  def test_CalcEMD2(self):
    cc = {}
    for r in [ 5, 10, 37, 38 ]:
      scratch = cv.CreateImage((100,100), 8, 1)
      cv.SetZero(scratch)
      cv.Circle(scratch, (50,50), r, 255, -1)
      storage = cv.CreateMemStorage()
      seq = cv.FindContours(scratch, storage, cv.CV_RETR_TREE, cv.CV_CHAIN_APPROX_SIMPLE)
      arr = cv.CreateMat(len(seq), 3, cv.CV_32FC1)
      for i,e in enumerate(seq):
        arr[i,0] = 1
        arr[i,1] = e[0]
        arr[i,2] = e[1]
      cc[r] = arr
    def myL1(A, B, D):
      return abs(A[0]-B[0]) + abs(A[1]-B[1])
    def myL2(A, B, D):
      return math.sqrt((A[0]-B[0])**2 + (A[1]-B[1])**2)
    def myC(A, B, D):
      return max(abs(A[0]-B[0]), abs(A[1]-B[1]))
    contours = set(cc.values())
    for c0 in contours:
      for c1 in contours:
        self.assert_(abs(cv.CalcEMD2(c0, c1, cv.CV_DIST_L1) - cv.CalcEMD2(c0, c1, cv.CV_DIST_USER, myL1)) < 1e-3)
        self.assert_(abs(cv.CalcEMD2(c0, c1, cv.CV_DIST_L2) - cv.CalcEMD2(c0, c1, cv.CV_DIST_USER, myL2)) < 1e-3)
        self.assert_(abs(cv.CalcEMD2(c0, c1, cv.CV_DIST_C) - cv.CalcEMD2(c0, c1, cv.CV_DIST_USER, myC)) < 1e-3)

  def test_FindContours(self):
    random.seed(0)

    storage = cv.CreateMemStorage()
    for trial in range(10):
      scratch = cv.CreateImage((800,800), 8, 1)
      cv.SetZero(scratch)
      def plot(center, radius, mode):
        cv.Circle(scratch, center, radius, mode, -1)
        if radius < 20:
          return 0
        else:
          newmode = 255 - mode
          subs = random.choice([1,2,3])
          if subs == 1:
            return [ plot(center, radius - 5, newmode) ]
          else:
            newradius = int({ 2: radius / 2, 3: radius / 2.3 }[subs] - 5)
            r = radius / 2
            ret = []
            for i in range(subs):
              th = i * (2 * math.pi) / subs
              ret.append(plot((int(center[0] + r * math.cos(th)), int(center[1] + r * math.sin(th))), newradius, newmode))
            return sorted(ret)

      actual = plot((400,400), 390, 255 )

      seq = cv.FindContours(scratch, storage, cv.CV_RETR_TREE, cv.CV_CHAIN_APPROX_SIMPLE)

      def traverse(s):
        if s == None:
          return 0
        else:
          self.assert_(abs(cv.ContourArea(s)) > 0.0)
          ((x,y),(w,h),th) = cv.MinAreaRect2(s, cv.CreateMemStorage())
          self.assert_(((w / h) - 1.0) < 0.01)
          self.assert_(abs(cv.ContourArea(s)) > 0.0)
          r = []
          while s:
            r.append(traverse(s.v_next()))
            s = s.h_next()
          return sorted(r)
      self.assert_(traverse(seq.v_next()) == actual)

  def test_ConvexHull2(self):
    # Draw a series of N-pointed stars, find contours, assert the contour is not convex,
    # assert the hull has N segments, assert that there are N convexity defects.

    def polar2xy(th, r):
      return (int(400 + r * math.cos(th)), int(400 + r * math.sin(th)))
    storage = cv.CreateMemStorage(0)
    for way in ['CvSeq', 'CvMat', 'list']:
      for points in range(3,20):
        scratch = cv.CreateImage((800,800), 8, 1)
        sides = 2 * points
        cv.FillPoly(scratch, [ [ polar2xy(i * 2 * math.pi / sides, [100,350][i&1]) for i in range(sides) ] ], 255)

        seq = cv.FindContours(scratch, storage, cv.CV_RETR_TREE, cv.CV_CHAIN_APPROX_SIMPLE)

        if way == 'CvSeq':
          # pts is a CvSeq
          pts = seq
        elif way == 'CvMat':
          # pts is a CvMat
          arr = cv.CreateMat(len(seq), 1, cv.CV_32SC2)
          for i,e in enumerate(seq):
            arr[i,0] = e
          pts = arr
        elif way == 'list':
          # pts is a list of 2-tuples
          pts = list(seq)
        else:
          assert False

        self.assert_(cv.CheckContourConvexity(pts) == 0)
        hull = cv.ConvexHull2(pts, storage, return_points = 1)
        self.assert_(cv.CheckContourConvexity(hull) == 1)
        self.assert_(len(hull) == points)

        if way in [ 'CvSeq', 'CvMat' ]:
          defects = cv.ConvexityDefects(pts, cv.ConvexHull2(pts, storage), storage)
          self.assert_(len([depth for (_,_,_,depth) in defects if (depth > 5)]) == points)

  def xxxtest_corners(self):
    a = cv.LoadImage("foo-mono.png", 0)
    cv.AdaptiveThreshold(a, a, 255, param1=5)
    scribble = cv.CreateImage(cv.GetSize(a), 8, 3)
    cv.CvtColor(a, scribble, cv.CV_GRAY2BGR)
    if 0:
      eig_image = cv.CreateImage(cv.GetSize(a), cv.IPL_DEPTH_32F, 1)
      temp_image = cv.CreateImage(cv.GetSize(a), cv.IPL_DEPTH_32F, 1)
      pts = cv.GoodFeaturesToTrack(a, eig_image, temp_image, 100, 0.04, 2, use_harris=1)
      for p in pts:
        cv.Circle( scribble, p, 1, cv.RGB(255,0,0), -1 )
      self.snap(scribble)
    canny = cv.CreateImage(cv.GetSize(a), 8, 1)
    cv.SubRS(a, 255, canny)
    self.snap(canny)
    li = cv.HoughLines2(canny,
                        cv.CreateMemStorage(),
                        cv.CV_HOUGH_STANDARD,
                        1,
                        math.pi/180,
                        60,
                        0,
                        0)
    for (rho,theta) in li:
      print rho,theta
      c = math.cos(theta)
      s = math.sin(theta)
      x0 = c*rho
      y0 = s*rho
      cv.Line(scribble,
              (x0 + 1000*(-s), y0 + 1000*c),
              (x0 + -1000*(-s), y0 - 1000*c),
              (0,255,0))
    self.snap(scribble)

  def xxx_test_CalcOpticalFlowBM(self):
    a = cv.LoadImage("ab/0.tiff", 0)

    if 0:
      # create b, just a shifted 2 pixels in X
      b = cv.CreateImage(cv.GetSize(a), 8, 1)
      m = cv.CreateMat(2, 3, cv.CV_32FC1)
      cv.SetZero(m)
      m[0,0] = 1
      m[1,1] = 1
      m[0,2] = 2
      cv.WarpAffine(a, b, m)
    else:
      b = cv.LoadImage("ab/1.tiff", 0)

    if 1:
      factor = 2
      for i in range(50):
        print i
        o0 = cv.LoadImage("again3_2245/%06d.tiff" % i, 1)
        o1 = cv.LoadImage("again3_2245/%06d.tiff" % (i+1), 1)
        a = cv.CreateImage((640,360), 8, 3)
        b = cv.CreateImage((640,360), 8, 3)
        cv.Resize(o0, a)
        cv.Resize(o1, b)
        am = cv.CreateImage(cv.GetSize(a), 8, 1)
        bm = cv.CreateImage(cv.GetSize(b), 8, 1)
        cv.CvtColor(a, am, cv.CV_RGB2GRAY)
        cv.CvtColor(b, bm, cv.CV_RGB2GRAY)
        fi = FrameInterpolator(am, bm)
        for k in range(factor):
          on = (i * factor) + k
          cv.SaveImage("/Users/jamesb/Desktop/foo/%06d.png" % on, fi.lerp(k / float(factor), a, b))
      return

    if 0:
      # Run FlowBM
      w,h = cv.GetSize(a)
      wv = (w - 6) / 8
      hv = (h - 6) / 8
      velx = cv.CreateMat(hv, wv, cv.CV_32FC1)
      vely = cv.CreateMat(hv, wv, cv.CV_32FC1)
      cv.CalcOpticalFlowBM(a, b, (6,6), (8,8), (32,32), 0, velx, vely)

      if 1:
        scribble = cv.CreateImage(cv.GetSize(a), 8, 3)
        cv.CvtColor(a, scribble, cv.CV_GRAY2BGR)
        for y in range(0,360, 4):
          for x in range(0,640, 4):
            cv.Line(scribble, (x, y), (x+velx[y,x], y + vely[y,x]), (0,255,0))
        cv.Line(a, (640/5,0), (640/5,480), 255)
        cv.Line(a, (0,360/5), (640,360/5), 255)
        self.snap(scribbe)
        return 0
        ivx = cv.CreateMat(h, w, cv.CV_32FC1)
        ivy = cv.CreateMat(h, w, cv.CV_32FC1)
        cv.Resize(velx, ivx)
        cv.Resize(vely, ivy)

      cv.ConvertScale(ivx, ivx, 0.5)
      cv.ConvertScale(ivy, ivy, 0.5)
    
    if 1:
      w,h = cv.GetSize(a)
      velx = cv.CreateMat(h, w, cv.CV_32FC1)
      vely = cv.CreateMat(h, w, cv.CV_32FC1)
      cv.CalcOpticalFlowLK(a, b, (7,7), velx, vely)

      for i in range(10):
        cv.Smooth(velx, velx, param1 = 7)
        cv.Smooth(vely, vely, param1 = 7)
      scribble = cv.CreateImage(cv.GetSize(a), 8, 3)
      cv.CvtColor(a, scribble, cv.CV_GRAY2BGR)
      for y in range(0, 360, 8):
        for x in range(0, 640, 8):
          cv.Line(scribble, (x, y), (x+velx[y,x], y + vely[y,x]), (0,255,0))
      self.snapL((a,scribble,b))
      ivx = velx
      ivy = vely

    offx = cv.CreateMat(h, w, cv.CV_32FC1)
    offy = cv.CreateMat(h, w, cv.CV_32FC1)
    for y in range(360):
      for x in range(640):
        offx[y,x] = x
        offy[y,x] = y

    x = cv.CreateMat(h, w, cv.CV_32FC1)
    y = cv.CreateMat(h, w, cv.CV_32FC1)
    d = cv.CreateImage(cv.GetSize(a), 8, 1)
    cv.ConvertScale(velx, x, 1.0)
    cv.ConvertScale(vely, y, 1.0)
    cv.Add(x, offx, x)
    cv.Add(y, offy, y)

    cv.Remap(b, d, x, y)
    cv.Merge(d, d, a, None, scribble)
    original = cv.CreateImage(cv.GetSize(a), 8, 3)
    cv.Merge(b, b, a, None, original)
    self.snapL((original, scribble))

  def snap(self, img):
    self.snapL([img])

  def snapL(self, L):
    for i,img in enumerate(L):
      cv.NamedWindow("snap-%d" % i, 1)
      cv.ShowImage("snap-%d" % i, img)
    cv.WaitKey()
    cv.DestroyAllWindows()

  def yield_line_image(self):
    src = cv.LoadImage("building.jpg", 0)
    dst = cv.CreateImage(cv.GetSize(src), 8, 1)
    cv.Canny(src, dst, 50, 200, 3)
    return dst

  def test_HoughLines2_STANDARD(self):
    li = cv.HoughLines2(self.yield_line_image(),
                        cv.CreateMemStorage(),
                        cv.CV_HOUGH_STANDARD,
                        1,
                        math.pi/180,
                        100,
                        0,
                        0)
    self.assert_(len(li) > 0)
    self.assert_(li[0] != None)

  def test_HoughLines2_PROBABILISTIC(self):
    li = cv.HoughLines2(self.yield_line_image(),
                        cv.CreateMemStorage(),
                        cv.CV_HOUGH_PROBABILISTIC,
                        1,
                        math.pi/180,
                        50,
                        50,
                        10)
    self.assert_(len(li) > 0)
    self.assert_(li[0] != None)

  def test_Save(self):
    for o in [ cv.CreateImage((128,128), cv.IPL_DEPTH_8U, 1), cv.CreateMat(16, 16, cv.CV_32FC1) ]:
      cv.Save("test.save", o)
      loaded = cv.Load("test.save", cv.CreateMemStorage())
      self.assert_(type(o) == type(loaded))

  def test_ExtractSURF(self):
    img = cv.LoadImage('Stu.jpg', 0)
    w,h = cv.GetSize(img)
    for hessthresh in [ 300,400,500]:
      for dsize in [0,1]:
        for layers in [1,3,10]:
          kp,desc = cv.ExtractSURF(img, None, cv.CreateMemStorage(), (dsize, hessthresh, 3, layers))
          self.assert_(len(kp) == len(desc))
          for d in desc:
            self.assert_(len(d) == {0:64, 1:128}[dsize])
          for pt,laplacian,size,dir,hessian in kp:
            self.assert_((0 <= pt[0]) and (pt[0] <= w))
            self.assert_((0 <= pt[1]) and (pt[1] <= h))
            self.assert_(laplacian in [-1, 0, 1])
            self.assert_((0 <= dir) and (dir <= 360))
            self.assert_(hessian >= hessthresh)

  def test_Haar(self):
    import os
    hcfile = os.environ['OPENCV_ROOT'] + '/share/opencv/haarcascades/haarcascade_frontalface_default.xml'
    hc = cv.Load(hcfile)
    img = cv.LoadImage('Stu.jpg', 0)
    faces = cv.HaarDetectObjects(img, hc, cv.CreateMemStorage())
    self.assert_(len(faces) > 0)
    for (x,y,w,h),n in faces:
      cv.Rectangle(img, (x,y), (x+w,y+h), 255)
    #self.snap(img)

  def test_FindChessboardCorners(self):
    im = cv.CreateImage((512,512), cv.IPL_DEPTH_8U, 1)
    cv.Set(im, 128)
    def xf(i,j, o):
      return ((96 + o) + 40 * i, (96 + o) + 40 * j)
    for i in range(8):
      for j in range(8):
        color = ((i ^ j) & 1) * 255
        cv.Rectangle(im, xf(i,j, 0), xf(i,j, 39), color, cv.CV_FILLED)
    status,corners = cv.FindChessboardCorners( im, (7,7) )
    self.assert_(status)
    self.assert_(len(corners) == (7 * 7))
    if 0:
      im3 = cv.CreateImage(cv.GetSize(im), cv.IPL_DEPTH_8U, 3)
      cv.Merge(im, im, im, None, im3)
      cv.DrawChessboardCorners(im3, (7,7), corners, status)
      self.snap(im3)

  def test_FillPoly(self):
    scribble = cv.CreateImage((640,480), cv.IPL_DEPTH_8U, 1)
    random.seed(0)
    for i in range(50):
      cv.SetZero(scribble)
      self.assert_(cv.CountNonZero(scribble) == 0)
      cv.FillPoly(scribble, [ [ (random.randrange(640), random.randrange(480)) for i in range(100) ] ], (255,))
      self.assert_(cv.CountNonZero(scribble) != 0)

  def test_create(self):
    """ CvCreateImage, CvCreateMat and the header-only form """
    for (w,h) in [ (320,400), (640,480), (1024, 768) ]:
      data = "z" * (w * h)

      im = cv.CreateImage((w,h), 8, 1)
      cv.SetData(im, data, w)
      im2 = cv.CreateImageHeader((w,h), 8, 1)
      cv.SetData(im2, data, w)
      self.assertSame(im, im2)

      m = cv.CreateMat(h, w, cv.CV_8UC1)
      cv.SetData(m, data, w)
      m2 = cv.CreateMatHeader(h, w, cv.CV_8UC1)
      cv.SetData(m2, data, w)
      self.assertSame(m, m2)

      self.assertSame(im, m)
      self.assertSame(im2, m2)

  def test_casts(self):
    """ Exercise GetMat and GetImage """
    im = cv.LoadImage("lena.jpg", 0)
    data = im.tostring()
    cv.SetData(im, data, cv.GetSize(im)[0])

    start_count = sys.getrefcount(data)

    # Conversions should produce same data
    self.assertSame(im, cv.GetImage(im))
    m = cv.GetMat(im)
    self.assertSame(im, m)
    self.assertSame(m, cv.GetImage(m))
    im2 = cv.GetImage(m)
    self.assertSame(im, im2)
    
    self.assertEqual(sys.getrefcount(data), start_count + 2)
    del im2
    self.assertEqual(sys.getrefcount(data), start_count + 1)
    del m
    self.assertEqual(sys.getrefcount(data), start_count)
    del im
    self.assertEqual(sys.getrefcount(data), start_count - 1)

  def temp_test(self):
    cv.temp_test()

  def play(self):

    print
    print "%" * 80
    print

    if 0:
      # XXX - email discuss with Vadim
      m = cv.CreateMat(1, 16, cv.CV_32FC1)
      print m
      m[0,1] = 0
      return

    if 0:
      src = cv.CreateImage((128,128), cv.IPL_DEPTH_8U, 1)
      dst = cv.CreateImage((128,128), cv.IPL_DEPTH_8U, 1)
      mapping = cv.CreateMat(2, 3, cv.CV_32FC1)
      cv.WarpAffine(src, dst, mapping, fillval = (8,4))

    if 0:
      # self.test_CreateMat()
      mat = cv.CreateMat(5, 5, cv.CV_32FC1)
      print "HERE", mat[3,2]
      #mat[3,2] += 0.787
      print mat[3,2]

    self.test_tostring()
    # self.test_leak()

    if 0:
      im = cv.LoadImage("foo.png")
      dst = cv.CreateImage(cv.GetSize(im), cv.IPL_DEPTH_16S, 3);
      cv.Laplace(im, dst)
      mini = cv.CreateImage(cv.GetSize(im), cv.IPL_DEPTH_8U, 3);
      cv.ConvertScale(dst, mini)
      cv.SaveImage("foo_laplace.png", mini)

    if 0:
      a = cv.CreateImage((128,128), cv.IPL_DEPTH_8U, 1);
      cv.SetData(a, " " * (128*132), 128)
      print a[3,3]
      a[3,3] = (6,)
      print a[3,3]
      cv.SaveImage("foo1.jpg", a)
      dst = cv.CreateImage(cv.GetSize(a), cv.IPL_DEPTH_16S, 1);
      cv.Laplace(a, dst)

    if 0:
      scribble = cv.CreateImage((640,480), cv.IPL_DEPTH_8U, 1)
      cv.SetZero(scribble)
      #cv.FillPoly(scribble, [ [ (10,10), (100,480), (300,40) ], [ (500,10), (510,13), (500,25) ] ], (255,))
      cv.FillPoly(scribble, [ [ (random.randrange(640), random.randrange(480)) for i in range(100) ] ], (255,))
      cv.SaveImage("scribble.png", scribble)
      print "Nonzero pixels:", cv.CountNonZero(scribble)
      print "AvgSdv:", cv.AvgSdv(scribble)

    if 0:
      md = cv.CreateMatND([640,480], cv.CV_8UC1)
      print md
      print cv.GetDims(md)
      #cv.SaveImage("klak.png", md)
      print md[639,479]

    if 0:
      src = cv.LoadImage('building.jpg', 0)
      eq = cv.CreateImage(cv.GetSize(src), 8, 1)
      cv.EqualizeHist(src, eq)
      cv.NamedWindow("before", 1)
      cv.ShowImage("before", src)
      cv.NamedWindow("eq", 1)
      cv.ShowImage("eq", eq)
      cv.WaitKey()

      src = cv.LoadImage('building.jpg')
      h_plane = cv.CreateImage(cv.GetSize(src), 8, 1)
      s_plane = cv.CreateImage(cv.GetSize(src), 8, 1)
      v_plane = cv.CreateImage(cv.GetSize(src), 8, 1)
      hsv = cv.CreateImage(cv.GetSize(src), 8, 3)
      cv.CvtColor(src, hsv, cv.CV_BGR2HSV)
      cv.CvtPixToPlane( hsv, h_plane, s_plane, v_plane, None);
      h_bins = 30
      s_bins = 32
      scale = 10
      hist = cv.CreateHist([ h_bins, s_bins ], cv.CV_HIST_ARRAY, [ (0,180), (0,255) ], 1)
      cv.CalcHist([ h_plane, s_plane ], hist, 0)
      (_, max_value, _, _) = cv.GetMinMaxHistValue(hist)

      hist_img = cv.CreateImage((h_bins*scale,s_bins*scale), 8, 3)
      for h in range(h_bins):
        for s in range(s_bins):
          bin_val = (cv.QueryHistValue_2D( hist, h, s ) / max_value) * 255
          intensity = int(bin_val)
          cv.Rectangle(hist_img,
                       (h*scale, s*scale),
                       ((h+1)*scale - 1, (s+1)*scale - 1),
                       (intensity, intensity, intensity),
                       cv.CV_FILLED)

      cv.NamedWindow( "Source", 1 );
      cv.ShowImage( "Source", src );
      cv.NamedWindow( "H-S Histogram", 1 );
      cv.ShowImage( "H-S Histogram", hist_img );
      cv.WaitKey()

    if 0:
      storage = cv.CreateMemStorage(0)

      cv.NamedWindow("scribble")
      cv.ShowImage("scribble", scribble)
      seq = cv.FindContours(scribble, storage, cv.CV_RETR_CCOMP, cv.CV_CHAIN_APPROX_SIMPLE)
      print "CV_32SC2", cv.CV_32SC2
      print "seq", seq

      dst = cv.CreateImage( cv.GetSize(scribble), 8, 3 )
      color = (255,128,128)
      contour = seq
      cv.DrawContours( dst, contour, color, color, 0, 1, 8 )
      print "seq.h_next", contour.h_next()
      print "seq.v_next", contour.v_next()
      color = (128,255,128)
      cv.DrawContours( dst, contour.v_next(), color, color, 1, 1, 8 )

      print "contour len", len(contour)
      print "First ", contour[0]
      print "First ten", contour[:10]
      print cv.CV_IS_SEQ_HOLE(contour)
      print cv.CV_IS_SEQ_HOLE(contour.v_next())

      cv.NamedWindow("contours")
      cv.ShowImage("contours", dst)

      cv.WaitKey()

    if 0:
      cv.NamedWindow("scribble")
      cv.ShowImage("scribble", scribble)
      def mycallback(event, x, y, flags, data):
        print event, x, y, flags, data
      cv.SetMouseCallback("scribble", mycallback, 'DATA')
      cv.WaitKey()

    if 0:
      eig_image = cv.CreateImage(cv.GetSize(scribble), cv.IPL_DEPTH_32F, 1)
      temp_image = cv.CreateImage(cv.GetSize(scribble), cv.IPL_DEPTH_32F, 1)
      for n in range(1,30):
        self.assert_(len(cv.GoodFeaturesToTrack(scribble, eig_image, temp_image, n, 0.04, 2)) <= n)

    print
    print "%" * 80

if __name__ == '__main__':
  random.seed(0)
  if 0:
    rostest.unitrun('visual_odometry', 'directed', TestDirected)
  else:
    suite = unittest.TestSuite()
    suite.addTest(TestDirected(sys.argv[1]))
    unittest.TextTestRunner(verbosity=2).run(suite)
