import random
import time
import unittest
import math
import sys

import cv

def by_response(kp):
    return sorted(kp, key = lambda x:(abs(x[2]),x[0],x[1]), reverse = True)
  
def simple(im, scales = 7, threshold = 30, line_threshold = 10, line_threshold_bin = 8):
    sizes = [4, 6, 8, 11, 16, 22, 23, 32, 45, 46, 64, 90, 128]
    storage = cv.CreateMemStorage()
    return by_response(cv.GetStarKeypoints(im, storage, ( sizes[scales], int(threshold), int(line_threshold), int(line_threshold_bin), 3)))

class TestGetStarKeypoints(unittest.TestCase):

    def setUp(self):
        self.im640 = cv.LoadImage("im640x480.pgm", 0)
        self.im64 = cv.CreateMat(10, 10, cv.CV_8UC1)
        cv.GetSubRect(self.im640, self.im64, (100, 100, 64, 64))

    def test_parameters(self):
        """ Change scales and thresholds """
        results = [
            simple(self.im640, 7, 30.0, 10.0, 8.0),
            simple(self.im640, 5, 30.0, 10.0, 8.0),
            simple(self.im640, 5, 25.0, 10.0, 8.0),
            simple(self.im640, 5, 25.0, 7.0, 8.0),
            simple(self.im640, 5, 25.0, 7.0, 4.0)
        ]
        # All should have different lengths
        self.assert_(len(set([ len(i) for i in results ])) == len(results))

    def test_large_sizes(self):
        sizes = [ 1024,1776,1777,1778,2000,8191,8192,8193]
        for s in sizes:
            simple(cv.CreateImage((s,100), 8, 1))
            simple(cv.CreateImage((100,s), 8, 1))

    def failing_test_small_sizes(self):
        sizes = [ 2,16,32,64,640 ]
        for s in sizes:
            simple(cv.CreateImage((s,100), 8, 1))
            simple(cv.CreateImage((100,s), 8, 1))

    def test_blank(self):
        """ Blank images have 0 keypoints """
        
        for xsize in [640]:
            for ysize in [480]:
                for c in [0,1,127,128,129,254,255]:
                    im = cv.CreateImage((xsize,ysize), 8, 1)
                    cv.Set(im, c)
                    self.assertEqual(simple(im, 7, 0.1), [])

    def test_r(self):
        """ Line of circles (varying r) across image each produces a strong response """

        im = cv.CreateImage((1000,200), 8, 1)
        npoints = 18
        xs = [ (100 + 50 * t) for t in range(npoints) ]
        for t in range(npoints):
            r = int(3.0+0.5*t)
            cv.Circle(im, (xs[t], 100), r, 255, -1)

        # Add noise into the image.  If the image does not contain noise,
        # then the non maximum suppression can - like Buridan's ass - be
        # presented with two adjacent responses that are equal, and reject
        # both because neither is a maximum.  The chance of this happening
        # with real-world images is very remote indeed.

        rand = cv.CreateImage(cv.GetSize(im), 8, 1)
        cv.RandArr(cv.RNG(), rand, cv.CV_RAND_UNI, 0, 8)
        cv.Add(im, rand, im)

        result = sorted(simple(im, 7, 1, 999999, 999999)[:npoints])

        # Must have npoints
        self.assertEqual(len(result), npoints)

        # X coordinates must be within 1 point of expected
        for i,((x,y),s,r) in enumerate(result):
            self.assert_(abs(x - xs[i]) <= 1)

        # Already ordered by x, so scale should be increasing
        ss = [s for ((x,y),s,r) in result]
        self.assertEqual(ss, sorted(ss))

    def test_L_symmetry(self):
        """
        image and negated image have the same keypoints
        """
        ref = simple(self.im640)
        self.assert_(len(ref) != 0) # if no hits in the image, have a test bug
        inv = cv.CreateImage(cv.GetSize(self.im640), 8, 1)
        cv.Not(self.im640, inv)
        result = simple(inv)
        self.assertEqual(len(ref), len(result))
        for (a,e) in zip(sorted(result), sorted(ref)):
            self.assertAlmostEqual(a[0][0], e[0][0], 3)
            self.assertAlmostEqual(a[0][1], e[0][1], 3)
            self.assertAlmostEqual(a[1], e[1], 3)
            self.assertAlmostEqual(a[2], -e[2], 3)

    def test_translation(self):
        """ Small target translated in center of image has the same keypoints """

        def xlat(x, y):
            im = cv.CreateImage((200,200), 8, 1)
            cv.SetZero(im)
            cv.SetImageROI(im, (x,y,64,64))
            cv.Copy(self.im64, im)
            cv.ResetImageROI(im)
            return im

        ref = [(x-70,y-70,s,r) for ((x,y),s,r) in simple(xlat(70, 70))]
        self.assert_(len(ref) != 0) # if no hits in the image, have a test bug
        for tx in [69,70,71,77]:
            for ty in [69,70,71,77]:
                result = simple(xlat(tx, ty))
                self.assertEqual(ref, [(x-tx,y-ty,s,r) for ((x,y),s,r) in result])

    def test_subimage(self):
        """ Small target's keypoints are independent of overall image size """

        def make_big(x, y):
            im = cv.CreateImage((x,y), 8, 1)
            cv.SetZero(im)
            cv.SetImageROI(im, (100,100,64,64))
            cv.Copy(self.im64, im)
            cv.ResetImageROI(im)
            return im

        ref = simple(make_big(300, 300))
        self.assert_(len(ref) != 0) # if no hits in the image, have a test bug
        for sx in [300,301,316]:
            for sy in [300,301,316]:
                im = make_big(sx, sy)
                result = simple(im)
                self.assertEqual(ref, result)

    def test_xy_symmetry(self):
        """
        Flipped,flopped,rotated images have the same number of keypoints
        """

        for dim in [(640,480), (255,256), (256,255)]:
            refim = cv.CreateImage(dim, 8, 1)
            cv.Resize(self.im640, refim)
            ref = simple(refim, 7, 30.0, 999999.0, 999999.0)
            self.assert_(len(ref) != 0) # if no hits in the image, have a test bug
            for f in [-1, 0, 1]:
                t = cv.CreateImage(dim, 8, 1)
                cv.Flip(refim, t, f)
                result = simple(t, 7, 30.0, 999999.0, 999999.0)
                self.assertEqual(len(ref), len(result))

    def test_all_scales(self):
        """ random circles all over screen - pass if all scales get hit """

        refim = cv.CreateImage((1000,1000), 8, 1)
        cv.SetZero(refim)
        random.seed(0)
        for t in range(100):
            cv.Circle(refim, (random.randrange(100, 900), random.randrange(100,900)), t / 2, 255, -1, cv.CV_AA)
        result = simple(refim, 7)
        self.assertEqual(set([s for ((x,y),s,r) in result]), set([4, 6, 8, 12, 16, 22]))

if __name__ == '__main__':
    suite = unittest.TestSuite()
    suite.addTest(TestGetStarKeypoints(sys.argv[1]))
    unittest.TextTestRunner(verbosity=2).run(suite)
